﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;

namespace ComparisSilverlightChallenge_Web
{
    /// <summary>
    /// Basic webservices to access the ComparisCarfinder database.
    /// </summary>
    /// <remarks>
    /// 1.) This sample code can be used, modified or removed for the competition.
    /// 2.) All methods accepting SQL statements should only be used for
    /// prototyping and testing. It is not recommended to use them in
    /// production.
    /// 3.) For security reasons, the ComparisCarfinder database files
    /// in App_Data are set to read-only in the file system.
    /// </remarks>
    public class CarfinderService : ICarfinderService
    {
        #region Ad

        /// <summary>
        /// Get a single Ad by ID.
        /// </summary>
        /// <param name="ID">The ID of the Ad.</param>
        /// <returns>Returns the Ad object with the specified ID.</returns>
        public Ad GetAd(int ID)
        {
            using (ComparisCarfinderDBDataContext carfinderDB = new ComparisCarfinderDBDataContext())
            {
                var ads = from a in carfinderDB.Ads where a.AdID == ID select a;
                return ads.First<Ad>();
            }
        }

        /// <summary>
        /// Get a list of Ads objects by sql query.
        /// </summary>
        /// <param name="sqlQuery">The sql query to run against the database.</param>
        /// <returns>Returns a list of Ad objects returned by the sql query.</returns>
        public List<Ad> GetAds(string sqlQuery)
        {
            using (ComparisCarfinderDBDataContext carfinderDB = new ComparisCarfinderDBDataContext())
            {
                var ads = carfinderDB.ExecuteQuery<Ad>(sqlQuery);
                return ads.ToList<Ad>();
            }
        }

        #endregion

        #region CarMake

        /// <summary>
        /// Get a single CarMake by ID.
        /// </summary>
        /// <param name="ID">The ID of the CarMake.</param>
        /// <returns>Returns the CarMake object with the specified ID.</returns>
        public CarMake GetCarMake(int ID)
        {
            using (ComparisCarfinderDBDataContext carfinderDB = new ComparisCarfinderDBDataContext())
            {
                var carMakes = from m in carfinderDB.CarMakes where m.CarMakeID == ID select m;
                return carMakes.First<CarMake>();
            }
        }

        /// <summary>
        /// Get a list of CarMake objects by sql query.
        /// </summary>
        /// <param name="sqlQuery">The sql query to run against the database.</param>
        /// <returns>Returns a list of CarMake objects returned by the sql query.</returns>
        public List<CarMake> GetCarMakes(string sqlQuery)
        {
            using (ComparisCarfinderDBDataContext carfinderDB = new ComparisCarfinderDBDataContext())
            {
                var carMakes = carfinderDB.ExecuteQuery<CarMake>(sqlQuery);
                return carMakes.ToList<CarMake>();
            }
        }

        #endregion

        #region CarModel

        /// <summary>
        /// Get a single CarModel by ID.
        /// </summary>
        /// <param name="ID">The ID of the CarModel.</param>
        /// <returns>Returns the GetCarModel object with the specified ID.</returns>
        public CarModel GetCarModel(int ID)
        {
            using (ComparisCarfinderDBDataContext carfinderDB = new ComparisCarfinderDBDataContext())
            {
                var carModels = from m in carfinderDB.CarModels where m.CarModelID == ID select m;
                return carModels.First<CarModel>();
            }
        }

        /// <summary>
        /// Get a list of CarModel objects by sql query.
        /// </summary>
        /// <param name="sqlQuery">The sql query to run against the database.</param>
        /// <returns>Returns a list of CarModel objects returned by the sql query.</returns>
        public List<CarModel> GetCarModels(string sqlQuery)
        {
            using (ComparisCarfinderDBDataContext carfinderDB = new ComparisCarfinderDBDataContext())
            {
                var carModels = carfinderDB.ExecuteQuery<CarModel>(sqlQuery);
                return carModels.ToList<CarModel>();
            }
        }

        #endregion

        #region Vendor

        /// <summary>
        /// Get a single Vendor by ID.
        /// </summary>
        /// <param name="ID">The ID of the Vendor.</param>
        /// <returns>Returns the Vendor object with the specified ID.</returns>
        public Vendor GetVendor(int ID)
        {
            using (ComparisCarfinderDBDataContext carfinderDB = new ComparisCarfinderDBDataContext())
            {
                var vendors = from v in carfinderDB.Vendors where v.VendorID == ID select v;
                return vendors.First<Vendor>();
            }
        }

        /// <summary>
        /// Get a list of Vendor objects by sql query.
        /// </summary>
        /// <param name="sqlQuery">The sql query to run against the database.</param>
        /// <returns>Returns a list of Vendor objects returned by the sql query.</returns>
        public List<Vendor> GetVendors(string sqlQuery)
        {
            using (ComparisCarfinderDBDataContext carfinderDB = new ComparisCarfinderDBDataContext())
            {
                var vendors = carfinderDB.ExecuteQuery<Vendor>(sqlQuery);
                return vendors.ToList<Vendor>();
            }
        }

        #endregion

        #region ExpandedAd

        /// <summary>
        /// Get a single ExpandedAd by ID.
        /// </summary>
        /// <param name="ID">The ID of the ExpandedAd.</param>
        /// <returns>Returns the ExpandedAd object with the specified ID.</returns>
        public ExpandedAd GetExpandedAd(int ID)
        {
            using (ComparisCarfinderDBDataContext carfinderDB = new ComparisCarfinderDBDataContext())
            {
                var ads = from a in carfinderDB.ExpandedAds where a.AdID == ID select a;
                return ads.First<ExpandedAd>();
            }
        }

        /// <summary>
        /// Get a list of ExpandedAd objects by sql query.
        /// </summary>
        /// <param name="sqlQuery">The sql query to run against the database.</param>
        /// <returns>Returns a list of ExpandedAd objects returned by the sql query.</returns>
        public List<ExpandedAd> GetExpandedAds(string sqlQuery)
        {
            using (ComparisCarfinderDBDataContext carfinderDB = new ComparisCarfinderDBDataContext())
            {
                var ads = carfinderDB.ExecuteQuery<ExpandedAd>(sqlQuery);
                return ads.ToList<ExpandedAd>();
            }
        }

        #endregion
    }
}
